
--[[--------------------------------------------------------------------
	Auc-Stat-TheUndermineJournal - The Undermine Journal price statistics module
	Copyright (c) 2011, 2012 Johnny C. Lam
	All rights reserved.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
	"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
	TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
	POSSIBILITY OF SUCH DAMAGE.
--]]--------------------------------------------------------------------

--[[
	Development has been resumed by Heavy Stone Crab in 2015.
--]]

if not AucAdvanced then return end

local libType, libName = "Stat", "The Undermine Journal"
local lib, parent, private = AucAdvanced.NewModule(libType, libName)
if not lib then return end

local aucPrint, decode, _, _, replicate, empty, get, set, default, debugPrint, fill, _TRANS = AucAdvanced.GetModuleLocals()
local GetFaction = AucAdvanced.GetFaction
local wipe = wipe

-- TUJ data table for market data from most-recently requested item.
local tujData = { }

lib.Processors = {
	config = function(callbackType, ...)
			private.SetupConfigGui(...)
		end,
	load = function(callbackType, addon)
			if private.OnLoad then
				private.OnLoad(addon)
			end
		end,
	tooltip = function(callbackType, ...)
			private.ProcessTooltip(...)
		end
}

-- lib.GetPrice()
--     (optional) Returns estimated price for item link.
function lib.GetPrice(hyperlink, serverKey)
	if not get("stat.underminejournal.enable") then return end
	if not private.GetInfo(hyperlink, serverKey) then return end
	return tujData.recent, tujData.market, tujData.stddev, tujData.globalMedian, tujData.globalMean,
		tujData.globalStdDev
end

-- lib.GetPriceColumns()
--     (optional) Returns the column names for GetPrice.
function lib.GetPriceColumns()
	return "3-Day Price", "14-Day Price", "Std Dev", "Global Median Price", "Global Mean Price",
		"Global Std Dev"
end

-- lib.GetPriceArray()
--     Returns pricing and other statistical info in an array.
local priceArray = { }
function lib.GetPriceArray(hyperlink, serverKey)
	if not get("stat.underminejournal.enable") then return end
	if not private.GetInfo(hyperlink, serverKey) then return end

	wipe(priceArray)

	local pricing = math.huge
	if get("stat.underminejournal.includeRecent") then
		if tujData.recent < pricing then
			pricing = tujData.recent
		end
	end
	if get("stat.underminejournal.includeMarket") then
		if tujData.market < pricing then
			pricing = tujData.market
		end
	end
	if get("stat.underminejournal.includeGlobalMedian") then
		if tujData.globalMedian < pricing then
			pricing = tujData.globalMedian
		end
	end
	if get("stat.underminejournal.includeGlobalMean") then
		if tujData.globalMean < pricing then
			pricing = tujData.globalMean
		end
	end
	-- this should be well-defined
	if not get("stat.underminejournal.includeStdDev") then
		if tujData.stddev > (tujData.market / 3) then
			pricing = 0
		end
	end
	-- this should be well-defined
	if not get("stat.underminejournal.includeGlobalStdDev") then
		if tujData.globalStdDev > (tujData.globalMedian / 3) then
			pricing = 0
		end
	end

	if pricing == math.huge then pricing = 0 end

	-- Quick fix; should be updated
	priceArray.price = pricing
	priceArray.seen = pricing
	priceArray.latest = pricing
	priceArray.market = pricing
	priceArray.median = pricing
	priceArray.stddev = pricing
	priceArray.cstddev = pricing
	priceArray.region_median = pricing
	priceArray.region_stddev = pricing
	priceArray.region_price = pricing
	priceArray.region_cstddev = pricing
	priceArray.qty = pricing

	return priceArray
end

-- lib.GetItemPDF()
--     Returns Probability Density Function for item link.
local bellCurve = AucAdvanced.API.GenerateBellCurve()
function lib.GetItemPDF(hyperlink, serverKey)
	if not get("stat.underminejournal.enable") then return end
	if not private.GetInfo(hyperlink, serverKey) then return end

	local mean, stddev = tujData.market, tujData.stddev
	if not mean or not stddev or stddev == 0 then
		-- No available data.
		return
	end

	-- Calculate the lower and upper bounds as +/- 3 standard deviations.
	local lower, upper = (mean - 3 * stddev), (mean + 3 * stddev)

	bellCurve:SetParameters(mean, stddev)
	return bellCurve, lower, upper
end

function private.OnLoad(addon)
	if addon ~= "auc-stat-theunderminejournal" then return end

	default("stat.underminejournal.enable", true)
	default("stat.underminejournal.includeRecent", false)
	default("stat.underminejournal.includeMarket", false)
	default("stat.underminejournal.includeStdDev", true)
	default("stat.underminejournal.includeGlobalMedian", true)
	default("stat.underminejournal.includeGlobalMean", false)
	default("stat.underminejournal.includeGlobalStdDev", true)
	default("stat.underminejournal.tooltip", true)
	default("stat.underminejournal.multiplyStackSize", false)
	--[[
	default("stat.underminejournal.input", true)
	default("stat.underminejournal.itemid", true)
	default("stat.underminejournal.bonuses", true)
	default("stat.underminejournal.age", true)
	]]--
	default("stat.underminejournal.recent", true)
	default("stat.underminejournal.market", true)
	default("stat.underminejournal.stddev", true)
	default("stat.underminejournal.globalMedian", true)
	default("stat.underminejournal.globalMean", true)
	default("stat.underminejournal.globalStdDev", true)
	default("stat.underminejournal.days", true)

	-- Only run this function once.
	private.OnLoad = nil
end

-- private.GetInfo(hyperlink, serverKey)
--     Returns the market info for the requested item in the tujData table.
function private.GetInfo(hyperlink, serverKey)
	local linkType, itemId, suffix, factor = decode(hyperlink)
	if linkType ~= "item" then return nil end

	wipe(tujData)

	if TUJMarketInfo then
		TUJMarketInfo(itemId, tujData)
	end
	if not tujData.itemid then
		return nil
	end

	return itemId
end

function private.SetupConfigGui(gui)
	local id = gui:AddTab(lib.libName, lib.libType .. " Modules")

	gui:AddHelp(id, "what the undermine journal",
		_TRANS('TUJ_Help_TheUndermineJournalQuestion1'),
		_TRANS('TUJ_Help_TheUndermineJournalAnswer1')
	)
	gui:AddHelp(id, "what auc stat the undermine journal",
		_TRANS('TUJ_Help_TheUndermineJournalQuestion2'),
		_TRANS('TUJ_Help_TheUndermineJournalAnswer2')
	)
	gui:AddHelp(id, "which setup",
		_TRANS('TUJ_Help_TheUndermineJournalQuestion3'),
		_TRANS('TUJ_Help_TheUndermineJournalAnswer3')
	)
	gui:AddHelp(id, "what setup",
		_TRANS('TUJ_Help_TheUndermineJournalQuestion4'),
		_TRANS('TUJ_Help_TheUndermineJournalAnswer4')
	)

	-- All options in here will be duplicated in the tooltip frame
	function private.addTooltipControls(id)
		gui:AddControl(id, "Header",     0,    _TRANS('TUJ_Interface_TheUndermineJournalOptions'))
		gui:AddControl(id, "Note",       0, 1, nil, nil, " ")

		gui:AddControl(id, "Checkbox",   0, 1, "stat.underminejournal.enable", _TRANS('TUJ_Interface_EnableTheUndermineJournal'))
		gui:AddTip(id, _TRANS('TUJ_HelpTooltip_EnableTheUndermineJournal'))
		gui:AddControl(id, "Note",       0, 1, nil, nil, " ")

		gui:AddControl(id, "Checkbox",   0, 4, "stat.underminejournal.includeRecent", _TRANS('TUJ_Interface_IncludeRecent'))
		gui:AddTip(id, _TRANS('TUJ_HelpTooltip_IncludeRecent'))
		gui:AddControl(id, "Checkbox",   0, 4, "stat.underminejournal.includeMarket", _TRANS('TUJ_Interface_IncludeMarket'))
		gui:AddTip(id, _TRANS('TUJ_HelpTooltip_IncludeMarket'))
		gui:AddControl(id, "Checkbox",   0, 4, "stat.underminejournal.includeStdDev", _TRANS('TUJ_Interface_IncludeStdDev'))
		gui:AddTip(id, _TRANS('TUJ_HelpTooltip_IncludeStdDev'))
		gui:AddControl(id, "Checkbox",   0, 4, "stat.underminejournal.includeGlobalMedian", _TRANS('TUJ_Interface_IncludeGlobalMedian'))
		gui:AddTip(id, _TRANS('TUJ_HelpTooltip_IncludeGlobalMedian'))
		gui:AddControl(id, "Checkbox",   0, 4, "stat.underminejournal.includeGlobalMean", _TRANS('TUJ_Interface_IncludeGlobalMean'))
		gui:AddTip(id, _TRANS('TUJ_HelpTooltip_IncludeGlobalMean'))
		gui:AddControl(id, "Checkbox",   0, 4, "stat.underminejournal.includeGlobalStdDev", _TRANS('TUJ_Interface_IncludeGlobalStdDev'))
		gui:AddTip(id, _TRANS('TUJ_HelpTooltip_IncludeGlobalStdDev'))
		gui:AddControl(id, "Note",       0, 1, nil, nil, " ")

		gui:AddControl(id, "Checkbox",   0, 4, "stat.underminejournal.tooltip", _TRANS('TUJ_Interface_ToggleTooltip'))
		gui:AddTip(id, _TRANS('TUJ_HelpTooltip_ToggleTooltip'))
		gui:AddControl(id, "Checkbox",   0, 6, "stat.underminejournal.multiplyStackSize", _TRANS('TUJ_Interface_ToggleMultiplyStackSize'))
		gui:AddTip(id, _TRANS('TUJ_HelpTooltip_ToggleMultiplyStackSize'))
		gui:AddControl(id, "Note",       0, 1, nil, nil, " ")
		gui:AddControl(id, "Checkbox",   0, 6, "stat.underminejournal.recent", _TRANS('TUJ_Interface_ToggleRecent'))
		gui:AddTip(id, _TRANS('TUJ_HelpTooltip_ToggleRecent'))
		gui:AddControl(id, "Checkbox",   0, 6, "stat.underminejournal.market", _TRANS('TUJ_Interface_ToggleMarket'))
		gui:AddTip(id, _TRANS('TUJ_HelpTooltip_ToggleMarket'))
		gui:AddControl(id, "Checkbox",   0, 6, "stat.underminejournal.stddev", _TRANS('TUJ_Interface_ToggleStdDev'))
		gui:AddTip(id, _TRANS('TUJ_HelpTooltip_ToggleStdDev'))
		gui:AddControl(id, "Checkbox",   0, 6, "stat.underminejournal.globalMedian", _TRANS('TUJ_Interface_ToggleGlobalMedian'))
		gui:AddTip(id, _TRANS('TUJ_HelpTooltip_ToggleGlobalMedian'))
		gui:AddControl(id, "Checkbox",   0, 6, "stat.underminejournal.globalMean", _TRANS('TUJ_Interface_ToggleGlobalMean'))
		gui:AddTip(id, _TRANS('TUJ_HelpTooltip_ToggleGlobalMean'))
		gui:AddControl(id, "Checkbox",   0, 6, "stat.underminejournal.globalStdDev", _TRANS('TUJ_Interface_ToggleGlobalStdDev'))
		gui:AddTip(id, _TRANS('TUJ_HelpTooltip_ToggleGlobalStdDev'))
		gui:AddControl(id, "Checkbox",   0, 6, "stat.underminejournal.days", _TRANS('TUJ_Interface_ToggleDays'))
		gui:AddTip(id, _TRANS('TUJ_HelpTooltip_ToggleDays'))
		gui:AddControl(id, "Note",       0, 1, nil, nil, " ")
	end

	local tooltipID = AucAdvanced.Settings.Gui.tooltipID

	private.addTooltipControls(id)
	if tooltipID then private.addTooltipControls(tooltipID) end
end

function private.ProcessTooltip(tooltip, name, hyperlink, quality, quantity, cost, ...)
	-- If the Auctioneer TUJ tooltip is enabled, then disable TUJ's tooltip, and vice versa.
	if not (get("stat.underminejournal.tooltip") and get("stat.underminejournal.enable")) then
		if TUJTooltip then TUJTooltip(true) end
		return
	else
		if TUJTooltip then TUJTooltip(false) end
	end
	local serverKey = GetFaction()
	if not private.GetInfo(hyperlink, serverKey) then return end

	if not quantity or quantity < 1 then quantity = 1 end
	local r1, g1, b1 = 0.4, 1, 0.4 -- 102, 255, 102
	local r2, g2, b2 = 0.9, 0.8, 0.5 -- The Undermine Journal

	if get("stat.underminejournal.multiplyStackSize") then
		tooltip:AddLine("  " .. _TRANS('TUJ_Tooltip_StackSize'):format(quantity), r1, g1, b1)
	else
		quantity = 1
	end
	if tujData.age then
		if tujData.age > (3 * 24 * 60 * 60) then
			tooltip:AddLine("  " .. _TRANS('TUJ_Tooltip_Age'):format(tujData.age / 24 / 60 / 60, (tujData.age / 24 / 60 / 60 - math.floor(tujData.age / 24 / 60 / 60)) * 24), r1, g1, b1)
		end
	end
	if tujData.recent and get("stat.underminejournal.recent") then
		tooltip:AddLine("  " .. _TRANS('TUJ_Tooltip_Recent'), tujData.recent * quantity)
	end
	if tujData.market and get("stat.underminejournal.market") then
		tooltip:AddLine("  " .. _TRANS('TUJ_Tooltip_Market'), tujData.market * quantity)
	end
	if tujData.stddev and get("stat.underminejournal.stddev") then
		tooltip:AddLine("  " .. _TRANS('TUJ_Tooltip_StdDev'), tujData.stddev * quantity, r2, g2, b2)
	end
	if tujData.globalMedian and get("stat.underminejournal.globalMedian") then
		tooltip:AddLine("  " .. _TRANS('TUJ_Tooltip_GlobalMedian'), tujData.globalMedian * quantity)
	end
	if tujData.globalMean and get("stat.underminejournal.globalMean") then
		tooltip:AddLine("  " .. _TRANS('TUJ_Tooltip_GlobalMean'), tujData.globalMean * quantity)
	end
	if tujData.globalStdDev and get("stat.underminejournal.globalStdDev") then
		tooltip:AddLine("  " .. _TRANS('TUJ_Tooltip_GlobalStdDev'), tujData.globalStdDev * quantity, r2, g2, b2)
	end
	if tujData.days and get("stat.underminejournal.days") then
		if tujData.days == 255 then
			tooltip:AddLine("  " .. _TRANS('TUJ_Tooltip_NotSeen'), r1, g1, b1)
		elseif tujData.days > 250 then
			tooltip:AddLine("  " .. _TRANS('TUJ_Tooltip_LastSeen250'), r1, g1, b1)
		elseif tujData.days >= 1 then
			tooltip:AddLine("  " .. _TRANS('TUJ_Tooltip_Days'):format(tujData.days),r1, g1, b1)
		end
	end
end
